@@ -22,6 +22,7 @@ contourmap <- function(mu,
                        ind,
                        levels,
                        type = c("standard",
+                                "pretty",
                                 "equalarea",
                                 "P0-optimal",
                                 "P1-optimal",
@@ -38,7 +39,6 @@ contourmap <- function(mu,
   type <- match.arg(type)
 
   if(missing(alpha) || is.null(alpha)){
-    cat("alpha not supplied, using alpha=0.1\n")
     alpha = 0.1
   }
   if(missing(F.limit)) {
@@ -46,6 +46,26 @@ contourmap <- function(mu,
   } else {
     F.limit = max(alpha,F.limit)
   }
+  if(missing(n.levels) || is.null(n.levels)){
+    if(missing(levels) || is.null(levels)){
+      stop("Must supply levels or n.levels")
+    } else {
+      n.levels = length(levels)
+    }
+  }
+
+  if(!missing(mu))
+    mu <- private.as.vector(mu)
+
+  if(!missing(vars))
+    vars <- private.as.vector(vars)
+
+  if(!missing(ind))
+    ind <- private.as.vector(ind)
+
+  if(!missing(Q))
+    Q <- private.as.Matrix(Q)
+
 
   measure = NULL
   if(!is.null(compute$measures))
@@ -59,6 +79,13 @@ contourmap <- function(mu,
     lp <- excursions.levelplot(mu=mu,n.levels = n.levels,ind = ind,
                                levels = levels,equal.area=FALSE)
   }
+  else if(type == 'pretty')
+  {
+    if(verbose) cat('Creating pretty contour map\n')
+    lp <- excursions.levelplot(mu=mu,n.levels = n.levels,ind = ind,
+                               levels = levels,equal.area=FALSE,pretty.cm=TRUE)
+    n.levels = lp$n.levels
+  }
   else if(type == 'equalarea')
   {
     if(verbose) cat('Creating equal area contour map\n')
@@ -114,7 +141,7 @@ contourmap <- function(mu,
           if(verbose) cat('Calculating P1-measure\n')
           lp$P1 <- Pmeasure(lp=lp,mu=mu,Q=Q,ind=ind,type=1)
         } else {
-          cat("Not computing P1-measure since it makes sense for n.levels>1\n")
+          lp$P1 = 1
         }
       } else if(measure[i] == "P2") {
         if(verbose) cat('Calculating P2-measure\n')
@@ -171,15 +198,17 @@ contourmap <- function(mu,
   } else {
     lp$E <- NULL
   }
-
   lp$meta <- list(calculation="contourmap",
                   F.limit=F.limit,
+                  F.computed = compute$F,
                   alpha=alpha,
                   levels=lp$u,
                   type="!=",
+                  contourmap.type = type,
                   n.iter=n.iter,
                   mu.range = range(mu[ind]),
-                  ind = ind)
+                  ind = ind,
+                  call = match.call())
   class(lp) <- "excurobj"
   return(lp)
 }
